'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var assert = _interopDefault(require('assert'));
var crypto = require('crypto');
var dgram = require('dgram');
var net = require('net');

(function (EnergySavingLevels) {
  EnergySavingLevels["auto"] = "auto";
  EnergySavingLevels["screenOff"] = "screenoff";
  EnergySavingLevels["maximum"] = "maximum";
  EnergySavingLevels["medium"] = "medium";
  EnergySavingLevels["minimum"] = "minimum";
  EnergySavingLevels["off"] = "off";
})(exports.EnergySavingLevels || (exports.EnergySavingLevels = {}));

(function (Inputs) {
  Inputs["dtv"] = "dtv";
  Inputs["atv"] = "atv";
  Inputs["cadtv"] = "cadtv";
  Inputs["catv"] = "catv";
  Inputs["av"] = "avav1";
  Inputs["component"] = "component1";
  Inputs["hdmi1"] = "hdmi1";
  Inputs["hdmi2"] = "hdmi2";
  Inputs["hdmi3"] = "hdmi3";
  Inputs["hdmi4"] = "hdmi4";
})(exports.Inputs || (exports.Inputs = {}));

(function (Keys) {
  Keys["arrowDown"] = "arrowdown";
  Keys["arrowLeft"] = "arrowleft";
  Keys["arrowRight"] = "arrowright";
  Keys["arrowUp"] = "arrowup";
  Keys["aspectRatio"] = "aspectratio";
  Keys["audioMode"] = "audiomode";
  Keys["back"] = "returnback";
  Keys["blueButton"] = "bluebutton";
  Keys["captionSubtitle"] = "captionsubtitle";
  Keys["channelDown"] = "channeldown";
  Keys["channelList"] = "channellist";
  Keys["channelUp"] = "channelup";
  Keys["deviceInput"] = "deviceinput";
  Keys["energySaving"] = "screenbright";
  Keys["fastForward"] = "fastforward";
  Keys["greenButton"] = "greenbutton";
  Keys["home"] = "myapp";
  Keys["info"] = "programminfo";
  Keys["liveTV"] = "livetv";
  Keys["menu"] = "settingmenu";
  Keys["number0"] = "number0";
  Keys["number1"] = "number1";
  Keys["number2"] = "number2";
  Keys["number3"] = "number3";
  Keys["number4"] = "number4";
  Keys["number5"] = "number5";
  Keys["number6"] = "number6";
  Keys["number7"] = "number7";
  Keys["number8"] = "number8";
  Keys["number9"] = "number9";
  Keys["ok"] = "ok";
  Keys["play"] = "play";
  Keys["previousChannel"] = "previouschannel";
  Keys["programGuide"] = "programguide";
  Keys["record"] = "record";
  Keys["redButton"] = "redbutton";
  Keys["rewind"] = "rewind";
  Keys["sleepTimer"] = "sleepreserve";
  Keys["userGuide"] = "userguide";
  Keys["videoMode"] = "videomode";
  Keys["volumeDown"] = "volumedown";
  Keys["volumeMute"] = "volumemute";
  Keys["volumeUp"] = "volumeup";
  Keys["yellowButton"] = "yellowbutton";
})(exports.Keys || (exports.Keys = {}));

const DefaultSettings = {
  encryptionIvLength: 16,
  encryptionKeyDigest: 'sha256',
  encryptionKeyIterations: 2 ** 14,
  encryptionKeyLength: 16,
  encryptionKeySalt: [0x63, 0x61, 0xb8, 0x0e, 0x9b, 0xdc, 0xa6, 0x63, 0x8d, 0x07, 0x20, 0xf2, 0xcc, 0x56, 0x8f, 0xb9],
  keycodeFormat: /[A-Z0-9]{8}/,
  messageBlockSize: 16,
  messageTerminator: '\r',
  networkPort: 9761,
  networkTimeout: 5000,
  networkWolAddress: '255.255.255.255',
  networkWolPort: 9
};

function assertSettings(settings) {
  assert(typeof settings === 'object' && settings !== null, 'settings must be an object');
  const {
    encryptionIvLength,
    encryptionKeyDigest,
    encryptionKeyIterations,
    encryptionKeyLength,
    encryptionKeySalt,
    keycodeFormat,
    messageBlockSize,
    messageTerminator
  } = settings;
  assert(typeof encryptionIvLength === 'number' && encryptionIvLength > 0, 'settings.encryptionIvLength must be a number greater than 0');
  assert(typeof encryptionKeyDigest === 'string' && encryptionKeyDigest.length > 0, 'settings.encryptionKeyDigest must be a string with length greater than 0');
  assert(typeof encryptionKeyIterations === 'number' && encryptionKeyIterations > 0, 'settings.encryptionKLeyIterations must be a number greater than 0');
  assert(typeof encryptionKeyLength === 'number' && encryptionKeyLength > 0, 'settings.encryptionKeyLength must be a number greater than 0');
  assert(Array.isArray(encryptionKeySalt) && encryptionKeySalt.some(data => typeof data === 'number' && data > 0), 'settings.encryptionKeySalt must be an array of numbers with length greater than 0');
  assert(keycodeFormat instanceof RegExp, 'settings.keycodeFormat must be an instance of RegExp');
  assert(typeof messageBlockSize === 'number' && messageBlockSize > 0, 'settings.messageBlockSize must be a number greater than 0');
  assert(typeof messageTerminator === 'string' && messageTerminator.length > 0, 'settings.messageTerminator must be a string with length greater than 0');
}

function deriveKey(keycode, settings = DefaultSettings) {
  assertSettings(settings);
  assert(typeof keycode === 'string', 'keycode must be a string');
  assert(settings.keycodeFormat.test(keycode), 'keycode format is invalid');
  return crypto.pbkdf2Sync(keycode, Buffer.from(settings.encryptionKeySalt), settings.encryptionKeyIterations, settings.encryptionKeyLength, settings.encryptionKeyDigest);
}

function generateRandomIv(length = DefaultSettings.encryptionIvLength) {
  assert(typeof length === 'number', 'length must be a number');
  assert(length > 0, 'length must be greater than 0');
  const iv = Buffer.alloc(length, 0);

  for (let i = 0; i < length; i++) {
    iv[i] = Math.floor(Math.random() * 255);
  }

  return iv;
}

class LGEncryption {
  constructor(keycode, settings = DefaultSettings) {
    this.settings = settings;
    assertSettings(settings);
    this.derivedKey = deriveKey(keycode, settings);
  }

  prepareMessage(message) {
    assert(typeof message === 'string', 'message must be a string');
    assert(message.length > 0, 'message must have a length greater than 0');
    const {
      messageTerminator,
      messageBlockSize
    } = this.settings;
    assert(!message.includes(messageTerminator), 'message must not include the message terminator character');
    let newMessage = message + messageTerminator;

    if (newMessage.length % messageBlockSize === 0) {
      newMessage += ' ';
    }

    const remainder = newMessage.length % messageBlockSize;

    if (remainder !== 0) {
      const padding = messageBlockSize - remainder;
      newMessage += String.fromCharCode(padding).repeat(padding);
    }

    return newMessage;
  }

  encrypt(message) {
    const iv = generateRandomIv(this.settings.encryptionKeyLength);
    const preparedMessage = this.prepareMessage(message);
    const ecbCypher = crypto.createCipheriv('aes-128-ecb', this.derivedKey, Buffer.alloc(0));
    const ivEnc = ecbCypher.update(iv);
    const cbcCypher = crypto.createCipheriv('aes-128-cbc', this.derivedKey, iv);
    const dataEnc = cbcCypher.update(preparedMessage, 'utf8');
    return Buffer.concat([ivEnc, dataEnc]);
  }

}

const MacAddressMatcher = /^([0-9a-f]{2}:){5}([0-9a-f]{2})$/i;
const MacAddressBlockSeparator = ':';
const MacAddressBlocksCount = 6;
const WolMacAddressCount = 16;
const WolSyncByte = 0xff;
const WolSyncCount = 6;

function assertSettings$1(settings) {
  assert(typeof settings === 'object' && settings !== null, 'settings must be an object');
  const {
    networkPort,
    networkTimeout,
    networkWolAddress,
    networkWolPort
  } = settings;
  assert(typeof networkPort === 'number' && networkPort > 0, 'settings.networkPort must be a number greater than 0');
  assert(typeof networkTimeout === 'number' && networkTimeout > 0, 'settings.networkTimeout must be a number greater than 0');
  assert(typeof networkWolAddress === 'string' && networkWolAddress.length > 0, 'settings.networkWolAddress must be a string with length greater than 0');
  assert(net.isIP(networkWolAddress), 'settings.networkWolAddress must be a valid IPv4 or IPv6');
  assert(typeof networkWolPort === 'number' && networkWolPort > 0, 'settings.networkWolPort must be a number greater than 0');
}

class TimeoutError extends Error {}
class TinySocket {
  constructor(host, macAddress, settings = DefaultSettings) {
    this.host = host;
    this.macAddress = macAddress;
    this.settings = settings;
    this.client = new net.Socket();
    assertSettings$1(settings);
    assert(macAddress === null || MacAddressMatcher.test(macAddress), 'invalid mac address');
  }

  wrap(method) {
    return new Promise((resolve, reject) => {
      const callback = (error, value) => {
        this.client.removeListener('error', reject);
        this.client.removeListener('timeout', () => {
          reject(new TimeoutError());
        });

        if (error) {
          reject(error);
        } else {
          resolve(value);
        }
      };

      method(callback);
      this.client.once('error', reject);
      this.client.once('timeout', reject);
    });
  }

  connect() {
    return this.wrap(callback => {
      this.client.connect(this.settings.networkPort, this.host, callback);
      this.client.setTimeout(this.settings.networkTimeout);
    });
  }

  handleTimeout() {
    this.client.end();
  }

  read() {
    return this.wrap(callback => {
      this.client.once('data', data => {
        callback(undefined, data);
      });
    });
  }

  write(data) {
    return this.wrap(callback => {
      this.client.write(data, callback);
    });
  }

  sendReceive(data) {
    try {
      const _this = this;

      return Promise.resolve(_this.write(data)).then(function () {
        return _this.read();
      });
    } catch (e) {
      return Promise.reject(e);
    }
  }

  disconnect() {
    return this.wrap(callback => {
      this.client.end(callback);
    });
  }

  wakeOnLan() {
    if (!this.macAddress) {
      throw new Error('Unable to wake on lan: mac address was not configured');
    }

    const socket = dgram.createSocket(net.isIPv6(this.settings.networkWolAddress) ? 'udp6' : 'udp4');
    socket.on('error', socket.close);
    socket.on('listening', () => {
      socket.setBroadcast(true);
    });
    const magicPacket = Buffer.alloc(WolSyncCount + MacAddressBlocksCount * WolMacAddressCount, WolSyncByte);
    const macAddressBlocks = this.macAddress.split(MacAddressBlockSeparator).map(string => parseInt(string, 16));

    for (let i = 0; i < WolMacAddressCount; i++) {
      for (let j = 0; j < macAddressBlocks.length; j++) {
        const index = WolSyncCount + i * MacAddressBlocksCount + j;
        magicPacket[index] = macAddressBlocks[j];
      }
    }

    socket.send(magicPacket, 0, magicPacket.length, this.settings.networkWolPort, this.settings.networkWolAddress, () => {
      socket.close();
    });
  }

}

const IntegerMatcher = /^\d+$/;
class LGTV {
  constructor(host, macAddress, keycode, settings = DefaultSettings) {
    this.socket = new TinySocket(host, macAddress, settings);
    this.encryption = new LGEncryption(keycode, settings);
  }

  sendCommand(command) {
    try {
      const _this = this;

      const encryptedData = _this.encryption.encrypt(command);

      return Promise.resolve(_this.socket.sendReceive(encryptedData)).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  }

  connect() {
    try {
      const _this2 = this;

      return Promise.resolve(_this2.socket.connect()).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  }

  disconnect() {
    try {
      const _this3 = this;

      return Promise.resolve(_this3.socket.disconnect()).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  }

  powerOff() {
    try {
      const _this4 = this;

      return Promise.resolve(_this4.sendCommand(`POWER off`)).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  }

  powerOn() {
    this.socket.wakeOnLan();
  }

  sendKey(key) {
    try {
      const _this5 = this;

      assert(Object.values(exports.Keys).some(availableKey => availableKey === key), 'key must be valid');
      return Promise.resolve(_this5.sendCommand(`KEY_ACTION ${key}`)).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  }

  setEnergySaving(level) {
    try {
      const _this6 = this;

      assert(Object.values(exports.EnergySavingLevels).some(availableOption => availableOption === level), 'level must be valid');
      return Promise.resolve(_this6.sendCommand(`ENERGY_SAVING ${level}`)).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  }

  setInput(input) {
    try {
      const _this7 = this;

      assert(Object.values(exports.Inputs).some(availableInput => availableInput === input), 'input must be valid');
      return Promise.resolve(_this7.sendCommand(`INPUT_SELECT ${input}`)).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  }

  setVolume(volumeLevel) {
    try {
      const _this8 = this;

      assert(typeof volumeLevel === 'number' && IntegerMatcher.test(volumeLevel.toString()) && volumeLevel >= 0 && volumeLevel <= 100, 'volumeLevel must be an integer between 0 and 100');
      return Promise.resolve(_this8.sendCommand(`VOLUME_CONTROL ${volumeLevel}`)).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  }

  setVolumeMute(isMuted) {
    try {
      const _this9 = this;

      assert(typeof isMuted === 'boolean', 'isMuted must be a boolean');
      return Promise.resolve(_this9.sendCommand(`VOLUME_MUTE ${isMuted ? 'on' : 'off'}`)).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  }

}

exports.DefaultSettings = DefaultSettings;
exports.LGTV = LGTV;
//# sourceMappingURL=lgtv-ip-control.cjs.development.js.map
